import { Observable, merge, BehaviorSubject } from 'rxjs';
import { DataSource } from '@angular/cdk/table';
import { EventEmitter } from '@angular/core';
import { map } from 'rxjs/operators';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
//import { ErrorDialog } from '../error/error-dialog.component';
/** @type {?} */
const TAG = 'TableDataSource';
/**
 * @abstract
 * @template E
 */
class TableDataSource extends DataSource {
    constructor() {
        //errorSubject: BehaviorSubject<Error> = new BehaviorSubject<Error>();
        super(...arguments);
        //statusEvent = new EventEmitter<string>();
        this.errorEvent = new EventEmitter();
        this.showSpinner = true;
    }
    /**
     *  Called when the table is being destroyed. Use this function, to clean up
     * any open connections or free any held resources that were set up during connect.
     * @return {?}
     */
    disconnect() {
        if (this.errorEvent)
            this.errorEvent.complete();
    }
    /**
     * overload to provide post load filtering
     * @protected
     * @param {?} data
     * @return {?}
     */
    // noinspection JSMethodCanBeStatic
    filterData(data) {
        return data;
    }
    // noinspection JSMethodCanBeStatic
    /**
     * @protected
     * @param {?} filter
     * @param {?} paginator
     * @return {?}
     */
    applyPaginator(filter, paginator) {
        if (filter && paginator) {
            filter.pager = {
                firstResult: (paginator.pageIndex || 0) * paginator.pageSize,
                pageSize: paginator.pageSize
            };
        }
    }
    // noinspection JSMethodCanBeStatic
    /**
     * @protected
     * @param {?} filter
     * @param {?} sort
     * @return {?}
     */
    applySort(filter, sort) {
        if (filter) {
            if (sort && sort.active) {
                filter.orderDir = sort.direction ? sort.direction.toUpperCase() : "ASC";
                filter.orderBy = sort.active;
            }
            if (filter.orderBy) {
                filter.orderBy = filter.orderBy.trim();
                if (filter.orderBy == '')
                    delete filter.orderBy;
            }
        }
    }
    // noinspection JSMethodCanBeStatic
    /**
     * @protected
     * @param {?} filter
     * @param {?=} value
     * @return {?}
     */
    applyKeyword(filter, value) {
        if (filter && value) {
            filter.keyword = value;
        }
    }
    /**
     * @protected
     * @param {?} e
     * @param {?=} op
     * @param {?=} component
     * @return {?}
     */
    onError(e, op, component = this.constructor.name) {
        this.status = 'error';
        e.error = Object.assign({ operation: op, component: component }, e.error);
        console.warn(TAG, 'Error loading data', op, e);
        this.errorEvent.emit(e);
    }
}
if (false) {
    /** @type {?} */
    TableDataSource.prototype.status;
    /** @type {?} */
    TableDataSource.prototype.errorEvent;
    /** @type {?} */
    TableDataSource.prototype.showSpinner;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
//import {FilteredTableSource} from "./filtered-table-source";
//import {finalize} from "rxjs/operators";
/** @type {?} */
const TAG$1 = 'MatTableSource';
/**
 * @abstract
 * @template E
 */
class MatTableSource extends TableDataSource {
    /**
     * @protected
     * @param {?=} paginator
     * @param {?=} sort
     */
    constructor(paginator, sort) {
        super();
        this.paginator = paginator;
        this.sort = sort;
        this.dataList = [];
        this.status = 'loading';
    }
    /**
     * @protected
     * @return {?} data mutations which trigger table reload
     */
    initReloadMutations() {
        /** @type {?} */
        const dataMutations = [];
        if (this.paginator)
            dataMutations.push(this.paginator.page);
        if (this.sort)
            dataMutations.push(this.sort.sortChange);
        return dataMutations;
    }
    /**
     * @protected
     * @param {?} totalRows total number of rows (without paging)
     * @return {?}
     */
    updatePager(totalRows) {
        if (this.paginator) {
            this.paginator.length = totalRows;
        }
    }
    /**
     * Connect this data source to the table. The table will only update when
     * the returned stream emits new items.
     * @return {?} A stream of the items to be rendered.
     */
    connect() {
        // Combine everything that affects the rendered data into one update
        // stream for the data-table to consume.
        /** @type {?} */
        const dataMutations = this.initReloadMutations();
        return new Observable((/**
         * @param {?} observer
         * @return {?}
         */
        observer => {
            merge(...dataMutations).subscribe((/**
             * @return {?}
             */
            () => {
                this.reloadTable().subscribe((/**
                 * @param {?} rows
                 * @return {?}
                 */
                rows => {
                    if (this.dataList.length == 0)
                        this.dataList = rows;
                    observer.next(this.filterData(rows));
                }));
            }));
        }));
    }
    /**
     * @protected
     * @param {?} data
     * @return {?}
     */
    pageData(data) {
        if (this.paginator != null) {
            this.paginator.length = data.length;
            /** @type {?} */
            const startIndex = this.paginator.pageIndex * this.paginator.pageSize;
            data = data.splice(startIndex, this.paginator.pageSize);
        }
        return data;
    }
    /**
     * @protected
     * @param {?} data
     * @return {?}
     */
    sortData(data) {
        if (!this.sort || !this.sort.active || this.sort.direction == '') {
            return data;
        }
        return data.sort((/**
         * @param {?} a
         * @param {?} b
         * @return {?}
         */
        (a, b) => {
            /** @type {?} */
            let propertyA;
            /** @type {?} */
            let propertyB;
            /** @type {?} */
            let prop = this.sort.active;
            //[propertyA, propertyB] = [a[prop], b[prop]];
            [propertyA, propertyB] = this.sortValues(a, b, prop);
            /** @type {?} */
            let valueA = isNaN(+propertyA) ? propertyA : +propertyA;
            /** @type {?} */
            let valueB = isNaN(+propertyB) ? propertyB : +propertyB;
            return (valueA < valueB ? -1 : 1) * (this.sort.direction == 'asc' ? 1 : -1);
        }));
    }
    /**
     * @protected
     * @param {?} a
     * @param {?} b
     * @param {?} prop
     * @return {?}
     */
    sortValues(a, b, prop) {
        return [a[prop], b[prop]];
    }
    /**
     * @protected
     * @param {?} data
     * @return {?}
     */
    searchData(data) {
        return data;
    }
}
if (false) {
    /**
     * @type {?}
     * @protected
     */
    MatTableSource.prototype.dataList;
    /**
     * @type {?}
     * @protected
     */
    MatTableSource.prototype.paginator;
    /**
     * @type {?}
     * @protected
     */
    MatTableSource.prototype.sort;
    /**
     * reload table data
     * @abstract
     * @protected
     * @return {?}
     */
    MatTableSource.prototype.reloadTable = function () { };
}
/**
 * @abstract
 * @template E
 */
class MixMatTableSource extends MatTableSource {
    /**
     * @protected
     * @param {?=} paginator
     * @param {?=} sort
     */
    constructor(paginator, sort) {
        super(paginator, sort);
        this.paginator = paginator;
        this.sort = sort;
        this.autoLoad = true;
        this.dataSubject = new BehaviorSubject(null);
    }
    /**
     * @return {?}
     */
    connect() {
        this.serverMutations = this.initReloadMutations();
        if (this.serverMutations.length > 0) {
            merge(...this.serverMutations).subscribe((/**
             * @return {?}
             */
            () => this.reload()));
        }
        else {
            console.debug(TAG$1, "connect: autoLoad", this.autoLoad);
            if (this.autoLoad) {
                this.reload();
            }
        }
        /** @type {?} */
        const dataMutations = this.initDisplayMutations();
        return new Observable((/**
         * @param {?} observer
         * @return {?}
         */
        observer => {
            merge(...dataMutations).subscribe((/**
             * @return {?}
             */
            () => {
                if (this.dataSubject.value)
                    observer.next(this.filterData(this.dataSubject.value.slice()));
            }));
        }));
    }
    /**
     * @return {?}
     */
    disconnect() {
        if (this.dataSubject != null)
            this.dataSubject.complete();
        super.disconnect();
    }
    /**
     * @protected
     * @param {?} data
     * @return {?}
     */
    filterData(data) {
        /** @type {?} */
        const origLength = data.length;
        /** @type {?} */
        const serverMutations = this.serverMutations;
        // if(this.filterSubject && serverMutations.indexOf(this.filterSubject) == -1)
        //   data = this.searchData(data);
        if (this.sort && serverMutations.indexOf(this.sort.sortChange) == -1)
            data = this.sortData(data);
        if (this.paginator && serverMutations.indexOf(this.paginator.page) == -1)
            data = this.pageData(data);
        console.debug(TAG$1, "filterData:", data.length, "/", origLength);
        return data;
    }
    /**
     * @return {?}
     */
    reload() {
        console.debug(TAG$1, "reload");
        this.status = 'loading';
        this.reloadTable()
            //.pipe(finalize(() => this.status = 'ready'))
            .subscribe((/**
         * @param {?} rows
         * @return {?}
         */
        rows => this.onLoaded(rows)), (/**
         * @param {?} error
         * @return {?}
         */
        error => this.onError(error, 'reload')));
    }
    /**
     * @protected
     * @param {?} rows
     * @return {?}
     */
    onLoaded(rows) {
        this.status = 'ready';
        this.dataSubject.next(rows);
    }
    /**
     * overload to setup server side mutations.
     * @protected
     * @return {?}
     */
    initReloadMutations() {
        return this.findAllMutations();
    }
    /**
     * @protected
     * @return {?}
     */
    findAllMutations() {
        return super.initReloadMutations();
    }
    /**
     * @protected
     * @return {?}
     */
    initClientMutations() {
        // remove server side mutations
        /** @type {?} */
        const remove = this.serverMutations;
        /** @type {?} */
        const dataMutations = this.findAllMutations();
        return dataMutations.filter((/**
         * @param {?} item
         * @return {?}
         */
        function (item) {
            return remove.indexOf(item) === -1;
        }));
    }
    /**
     * @protected
     * @return {?}
     */
    initDisplayMutations() {
        /** @type {?} */
        let dataMutations = [this.dataSubject];
        dataMutations = dataMutations.concat(this.initClientMutations());
        return dataMutations;
    }
    // protected applyFilter(filter : F) {
    //   if(this.paginator && this.serverMutations.indexOf(this.paginator.page) != -1)
    //     this.applyPaginator(filter, this.paginator);
    //   if(this.sort && this.serverMutations.indexOf(this.sort.sortChange) != -1)
    //     this.applySort(filter, this.sort);
    //   this.applyKeyword(filter);
    // }
    /**
     * @protected
     * @param {?} filter
     * @param {?} paginator
     * @return {?}
     */
    applyPaginator(filter, paginator) {
        if (paginator && this.serverMutations.indexOf(paginator.page) != -1)
            super.applyPaginator(filter, paginator);
    }
    /**
     * @protected
     * @param {?} filter
     * @param {?} sort
     * @return {?}
     */
    applySort(filter, sort) {
        if (sort && this.serverMutations.indexOf(sort.sortChange) != -1)
            super.applySort(filter, sort);
    }
    /**
     * @param {?} paginator
     * @return {?}
     */
    setPaginator(paginator) {
        this.paginator = paginator;
    }
    /**
     * @param {?} sort
     * @return {?}
     */
    setSort(sort) {
        this.sort = sort;
    }
}
if (false) {
    /** @type {?} */
    MixMatTableSource.prototype.dataSubject;
    /**
     * @type {?}
     * @protected
     */
    MixMatTableSource.prototype.serverMutations;
    /**
     * @type {?}
     * @protected
     */
    MixMatTableSource.prototype.autoLoad;
    /**
     * @type {?}
     * @protected
     */
    MixMatTableSource.prototype.paginator;
    /**
     * @type {?}
     * @protected
     */
    MixMatTableSource.prototype.sort;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const TAG$2 = 'FilteredTableSource';
/**
 * @abstract
 * @template F, E
 */
class FilteredTableSource extends MixMatTableSource {
    /**
     * @protected
     * @param {?=} paginator
     * @param {?=} sort
     * @param {?=} tableFilter
     */
    constructor(paginator, sort, tableFilter) {
        super(paginator, sort);
        this.paginator = paginator;
        this.sort = sort;
        this.tableFilter = tableFilter;
        this.filterSubject = new BehaviorSubject(tableFilter);
    }
    /**
     * @return {?}
     */
    disconnect() {
        if (this.filterSubject != null)
            this.filterSubject.complete();
        super.disconnect();
    }
    /**
     * @protected
     * @param {?} filter
     * @return {?}
     */
    applyFilter(filter) {
        this.applyPaginator(filter, this.paginator);
        this.applySort(filter, this.sort);
        this.applyKeyword(filter);
    }
    /**
     * @protected
     * @return {?}
     */
    findAllMutations() {
        /** @type {?} */
        let mutations = super.findAllMutations();
        if (this.filterSubject)
            mutations.push(this.filterSubject);
        //console.debug(TAG, "findAllMutations: mutations", mutations.length);
        return mutations;
    }
    /**
     * @protected
     * @return {?}
     */
    findFilter() {
        /** @type {?} */
        let filter = this.filterSubject.value;
        if (!filter) {
            filter = (/** @type {?} */ ({}));
        }
        return filter;
    }
    /**
     * @protected
     * @param {?} data
     * @return {?}
     */
    filterData(data) {
        /** @type {?} */
        const serverMutations = this.serverMutations;
        //console.debug(TAG, "filterData: serverMutations", serverMutations);
        if (this.filterSubject && serverMutations.indexOf(this.filterSubject) == -1)
            data = this.searchData(data);
        return super.filterData(data);
    }
}
if (false) {
    /** @type {?} */
    FilteredTableSource.prototype.filterSubject;
    /**
     * @type {?}
     * @protected
     */
    FilteredTableSource.prototype.paginator;
    /**
     * @type {?}
     * @protected
     */
    FilteredTableSource.prototype.sort;
    /**
     * @type {?}
     * @protected
     */
    FilteredTableSource.prototype.tableFilter;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
//import {MatPaginator} from "@angular/material/paginator";
//import {MatSort} from "@angular/material/sort";
/** @type {?} */
const TAG$3 = 'QueryTableSource';
/**
 * Data source for QueryResult objects
 * (including sorting, pagination, and filtering).
 * @abstract
 * @template F, E
 */
class QueryTableSource extends FilteredTableSource {
    /**
     * @protected
     * @return {?}
     */
    reloadTable() {
        return this.queryTable().pipe(map((/**
         * @param {?} qr
         * @return {?}
         */
        qr => {
            this.updatePager(qr.rows);
            return qr.items;
        })));
    }
}
if (false) {
    /**
     * query table data
     * @abstract
     * @protected
     * @return {?}
     */
    QueryTableSource.prototype.queryTable = function () { };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const TAG$4 = 'SearchTableSource';
/**
 * Data source for SearchResult objects
 * (including sorting, pagination, and filtering).
 * @abstract
 * @template F, E
 */
class SearchTableSource extends FilteredTableSource {
    /**
     * @protected
     * @return {?}
     */
    reloadTable() {
        return this.searchTable().pipe(map((/**
         * @param {?} qr
         * @return {?}
         */
        qr => {
            this.updatePager(qr.total);
            return qr.rows;
        })));
    }
}
if (false) {
    /**
     * query table data
     * @abstract
     * @protected
     * @return {?}
     */
    SearchTableSource.prototype.searchTable = function () { };
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,constantProperty,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { FilteredTableSource, MatTableSource, MixMatTableSource, QueryTableSource, SearchTableSource, TableDataSource };
//# sourceMappingURL=yukawa-chain-base-angular-store.js.map
